<?php

namespace Drupal\twig_extensions\Twig;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\StringTranslation\TranslationInterface;
use Twig\Extension\AbstractExtension;
use Twig\TwigFilter;

/**
 * Provides date manipulation filters.
 */
class DateExtension extends AbstractExtension {

  /**
   * The string translation service.
   *
   * @var \Drupal\Core\StringTranslation\TranslationInterface
   */
  protected $stringTranslation;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Constructs a new TimeDiffExtension object.
   *
   * @param \Drupal\Core\StringTranslation\TranslationInterface $stringTranslation
   *   The string translation service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(TranslationInterface $stringTranslation, TimeInterface $time) {
    $this->stringTranslation = $stringTranslation;
    $this->time = $time;
  }

  /**
   * {@inheritdoc}
   */
  public function getFilters(): array {
    return [
      new TwigFilter('time_diff', [$this, 'calculateTimeDiff']),
    ];
  }

  /**
   * Calculate the difference between the current time and the given timestamp.
   *
   * @param int|null $timestamp
   *   The timestamp to compare against the current time.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The formatted time difference.
   */
  public function calculateTimeDiff(?int $timestamp): TranslatableMarkup {
    $currentTime = $this->time->getCurrentTime();
    $diff = $currentTime - $timestamp;

    if ($diff < 0) {
      $diff = abs($diff);

      // Less than 1 minute.
      if ($diff < 60) {
        return $this->stringTranslation->formatPlural($diff, 'in 1 second', 'in @count seconds');
      }

      // Less than 1 hour.
      if ($diff < 3600) {
        $minutes = floor($diff / 60);
        return $this->stringTranslation->formatPlural($minutes, 'in 1 minute', 'in @count minutes');
      }

      // Less than 24 hours.
      if ($diff < 86400) {
        $hours = floor($diff / 3600);
        return $this->stringTranslation->formatPlural($hours, 'in 1 hour', 'in @count hours');
      }

      // Less than 7 days.
      if ($diff < 604800) {
        $days = floor($diff / 86400);
        return $this->stringTranslation->formatPlural($days, 'in 1 day', 'in @count days');
      }

      // Less than 30 days.
      if ($diff < 2592000) {
        $weeks = floor($diff / 604800);
        return $this->stringTranslation->formatPlural($weeks, 'in 1 week', 'in @count weeks');
      }

      // Less than 1 year.
      if ($diff < 31536000) {
        $months = floor($diff / 2592000);
        return $this->stringTranslation->formatPlural($months, 'in 1 month', 'in @count months');
      }

      // Greater than or equal to 1 year.
      $years = floor($diff / 31536000);
      return $this->stringTranslation->formatPlural($years, 'in 1 year', 'in @count years');
    }

    if ($diff === 0) {
      return $this->stringTranslation->translate('now');
    }

    // Less than 1 minute.
    if ($diff < 60) {
      return $this->stringTranslation->formatPlural($diff, '1 second ago', '@count seconds ago');
    }

    // Less than 1 hour.
    if ($diff < 3600) {
      $minutes = floor($diff / 60);
      return $this->stringTranslation->formatPlural($minutes, '1 minute ago', '@count minutes ago');
    }

    // Less than 24 hours.
    if ($diff < 86400) {
      $hours = floor($diff / 3600);
      return $this->stringTranslation->formatPlural($hours, '1 hour ago', '@count hours ago');
    }

    // Less than 7 days.
    if ($diff < 604800) {
      $days = floor($diff / 86400);
      return $this->stringTranslation->formatPlural($days, '1 day ago', '@count days ago');
    }

    // Less than 30 days.
    if ($diff < 2592000) {
      $weeks = floor($diff / 604800);
      return $this->stringTranslation->formatPlural($weeks, '1 week ago', '@count weeks ago');
    }

    // Less than 1 year.
    if ($diff < 31536000) {
      $months = floor($diff / 2592000);
      return $this->stringTranslation->formatPlural($months, '1 month ago', '@count months ago');
    }

    // Greater than 1 year.
    $years = floor($diff / 31536000);
    return $this->stringTranslation->formatPlural($years, '1 year ago', '@count years ago');
  }

}
