<?php

namespace Drupal\twig_extensions\Twig;

use Twig\Environment;
use Twig\Error\SyntaxError;
use Twig\Extension\AbstractExtension;
use Twig\Extension\CoreExtension;
use Twig\TwigFilter;

/**
 * Provides internationalization filters and functions.
 *
 * This class is copied and adapted from the twig/extensions package.
 */
class IntlExtension extends AbstractExtension {

  /**
   * {@inheritdoc}
   */
  public function getFilters(): array {
    return [
      new TwigFilter('localizeddate', [$this, 'localizedDateFilter'], ['needs_environment' => TRUE]),
      new TwigFilter('localizednumber', [$this, 'localizedNumberFilter']),
      new TwigFilter('localizedcurrency', [$this, 'localizedCurrencyFilter']),
    ];
  }

  /**
   * Formats a date according to the given locale and formats.
   */
  public function localizedDateFilter(Environment $env, $date, $dateFormat = 'medium', $timeFormat = 'medium', $locale = NULL, $timezone = NULL, $format = NULL, $calendar = 'gregorian') {
    $this->ensureIntlExtension();

    if (function_exists('twig_date_converter')) {
      $date = \twig_date_converter($env, $date, $timezone);
    }
    elseif (!($date instanceof \DateTime)) {
      $env->getExtension(CoreExtension::class)->convertDate($date, $timezone);
    }

    $formatValues = [
      'none' => \IntlDateFormatter::NONE,
      'short' => \IntlDateFormatter::SHORT,
      'medium' => \IntlDateFormatter::MEDIUM,
      'long' => \IntlDateFormatter::LONG,
      'full' => \IntlDateFormatter::FULL,
    ];

    if (!class_exists('IntlTimeZone')) {
      $formatter = \IntlDateFormatter::create(
            $locale,
            $formatValues[$dateFormat],
            $formatValues[$timeFormat],
            $date->getTimezone()->getName(),
            'gregorian' === $calendar ? \IntlDateFormatter::GREGORIAN : \IntlDateFormatter::TRADITIONAL,
            $format
        );

      return $formatter->format($date->getTimestamp());
    }

    $formatter = \IntlDateFormatter::create(
          $locale,
          $formatValues[$dateFormat],
          $formatValues[$timeFormat],
          \IntlTimeZone::createTimeZone($date->getTimezone()->getName()),
          'gregorian' === $calendar ? \IntlDateFormatter::GREGORIAN : \IntlDateFormatter::TRADITIONAL,
          $format
      );

    return $formatter->format($date->getTimestamp());
  }

  /**
   * Formats a number according to the given locale, style, and type.
   */
  public function localizedNumberFilter($number, $style = 'decimal', $type = 'default', $locale = NULL) {
    $this->ensureIntlExtension();

    static $typeValues = [
      'default' => \NumberFormatter::TYPE_DEFAULT,
      'int32' => \NumberFormatter::TYPE_INT32,
      'int64' => \NumberFormatter::TYPE_INT64,
      'double' => \NumberFormatter::TYPE_DOUBLE,
      'currency' => \NumberFormatter::TYPE_CURRENCY,
    ];

    $formatter = $this->getNumberFormatter($locale, $style);

    if (!isset($typeValues[$type])) {
      throw new SyntaxError(sprintf('The type "%s" does not exist. Known types are: "%s"', $type, implode('", "', array_keys($typeValues))));
    }

    return $formatter->format($number, $typeValues[$type]);
  }

  /**
   * Formats a number as a localized currency string.
   */
  public function localizedCurrencyFilter($number, $currency = NULL, $locale = NULL) {
    $this->ensureIntlExtension();

    $formatter = $this->getNumberFormatter($locale, 'currency');

    return $formatter->formatCurrency($number, $currency);
  }

  /**
   * Gets a number formatter instance according to given locale and formatter.
   *
   * @param string|null $locale
   *   Locale in which the number would be formatted.
   * @param string|null $style
   *   Style of the formatting.
   *
   * @return \NumberFormatter
   *   A NumberFormatter instance
   */
  protected function getNumberFormatter(?string $locale, ?string $style): \NumberFormatter {
    static $formatter, $currentStyle;

    $locale = $locale ?? \Locale::getDefault();

    if ($formatter && $formatter->getLocale() === $locale && $currentStyle === $style) {
      // Return same instance of NumberFormatter if parameters are the same
      // to those in previous call.
      return $formatter;
    }

    static $styleValues = [
      'decimal' => \NumberFormatter::DECIMAL,
      'currency' => \NumberFormatter::CURRENCY,
      'percent' => \NumberFormatter::PERCENT,
      'scientific' => \NumberFormatter::SCIENTIFIC,
      'spellout' => \NumberFormatter::SPELLOUT,
      'ordinal' => \NumberFormatter::ORDINAL,
      'duration' => \NumberFormatter::DURATION,
    ];

    if (!isset($styleValues[$style])) {
      throw new SyntaxError(sprintf('The style "%s" does not exist. Known styles are: "%s"', $style, implode('", "', array_keys($styleValues))));
    }

    $currentStyle = $style;

    return \NumberFormatter::create($locale, $styleValues[$style]);
  }

  /**
   * Ensures that the intl extension is available.
   *
   * @throws \RuntimeException
   *   If the intl extension is not available.
   */
  protected function ensureIntlExtension() {
    if (!class_exists(\IntlDateFormatter::class)) {
      throw new \RuntimeException('The native PHP intl extension (http://php.net/manual/en/book.intl.php) is needed to use intl-based filters.');
    }
  }

}
