<?php

namespace Drupal\twig_extensions\Twig;

use Twig\Environment;
use Twig\Extension\AbstractExtension;
use Twig\TwigFilter;

/**
 * Provides array manipulation filters.
 *
 * This class is copied and adapted from the twig/extensions package.
 */
class TextExtension extends AbstractExtension {

  /**
   * {@inheritdoc}
   */
  public function getFilters(): array {
    return [
      new TwigFilter('truncate', [$this, 'truncate'], ['needs_environment' => TRUE]),
      new TwigFilter('wordwrap', [$this, 'wordwrap'], ['needs_environment' => TRUE]),
    ];
  }

  /**
   * Truncates a string to a given length.
   *
   * @param \Twig\Environment $env
   *   The Twig environment.
   * @param string|null $value
   *   The text to truncate.
   * @param int $length
   *   The maximum length of the truncated text.
   * @param bool $preserve
   *   Whether to preserve whole words when truncating.
   * @param string $separator
   *   The separator to append at the end of the truncated text.
   *
   * @return string
   *   The truncated text.
   */
  public function truncate(Environment $env, ?string $value, int $length = 30, bool $preserve = false, string $separator = '...'): string {
    if (!is_string($value)) {
      return '';
    }

    if (mb_strlen($value, $env->getCharset()) > $length) {
      if ($preserve) {
        // If breakpoint is on the last word, return the value without separator.
        if (FALSE === ($breakpoint = mb_strpos($value, ' ', $length, $env->getCharset()))) {
          return $value;
        }

        $length = $breakpoint;
      }

      return rtrim(mb_substr($value, 0, $length, $env->getCharset())) . $separator;
    }

    return $value;
  }

  /**
   * Wraps text into new lines when it reaches a specific length.
   *
   * @param \Twig\Environment $env
   *   The Twig environment.
   * @param string|null $value
   *   The text to wrap.
   * @param int $length
   *   The maximum line length.
   * @param string $separator
   *   The separator to use for wrapping.
   * @param bool $preserve
   *   Whether to preserve the original line breaks.
   *
   * @return string
   *   The wrapped text.
   */
  public function wordwrap(Environment $env, ?string $value, int $length = 80, string $separator = "\n", bool $preserve = FALSE): string {
    if (!is_string($value)) {
      return '';
    }

    $sentences = [];

    $previous = mb_regex_encoding();
    mb_regex_encoding($env->getCharset());

    $pieces = mb_split($separator, $value);
    mb_regex_encoding($previous);

    foreach ($pieces as $piece) {
      while (!$preserve && mb_strlen($piece, $env->getCharset()) > $length) {
        $sentences[] = mb_substr($piece, 0, $length, $env->getCharset());
        $piece = mb_substr($piece, $length, 2048, $env->getCharset());
      }

      $sentences[] = $piece;
    }

    return implode($separator, $sentences);
  }

}
